

************************************************************************;
* Produce Thunderstorm Scatter Plots Using SAS/GRAPH and ODS Graphics	;
* Author: Charlie Liu													;
************************************************************************;

options mprint symbolgen nodate nonumber validvarname=v7;
%let pgmname = Thunderstorm Scatter Plots.sas;
%let outloc = C:\SAS Graphs;
%let pgmpth = &outloc.\&pgmname. &sysdate9. &systime. SAS V&sysver.;


** set site number;
%let sitenum = 10;
%let seed = 1699; 
%let SD = 3.5;

proc format;
  value visdf
    1 = 'Baseline'
    2 = 'Week 4'
    3 = 'Week 8'
    4 = 'Week 12';

  value hrdf
	1 = 'Hour 0'
	2 = 'Hour 2'
	3 = 'Hour 8';
run;


** Generate the required number of subjects;
data site_subj;
  do i = 1 to &sitenum.;
    do j = 1 to 15;
    siteid = i;
	subjid = i * 1000 + j; ** Subject id begin with 1001 at site 1 and with 2001 at site 2 etc.;	
	shuffle = ranuni (&seed. + i + j);  ** Randomly assign a shuffle number to each subject;
    output;
	end;
  end;
  drop i j; 
run;

proc sort data=site_subj;
  by shuffle;
run;

** Drop 50 subjects so each site might have different subject numbers;
data site_subj2;
  set site_subj;
  if _N_ <= 50 then delete;
run;

proc sort data=site_subj2;
  by siteid subjid;
run;

** Distribution of subjects by site;
proc freq data=site_subj2 noprint;
  table siteid/out=subj_freq (drop=PERCENT);
run;

** Save the subject number at each site to macro variables to be used later-on;
data _NULL_;
  set subj_freq;
  if siteid = 1 then call symput ('n_site1', put(count, 3.));
  if siteid = 2 then call symput ('n_site2', put(count, 3.));
  if siteid = 3 then call symput ('n_site3', put(count, 3.));
  if siteid = 4 then call symput ('n_site4', put(count, 3.));
  if siteid = 5 then call symput ('n_site5', put(count, 3.));
  if siteid = 6 then call symput ('n_site6', put(count, 3.));
  if siteid = 7 then call symput ('n_site7', put(count, 3.));
  if siteid = 8 then call symput ('n_site8', put(count, 3.));
  if siteid = 9 then call symput ('n_site9', put(count, 3.));
  if siteid = 10 then call symput ('n_site10', put(count, 3.));
run;


** Set up the study eye IOP values per subject based on visits/timepoints: assuming 6 to 8 mmHg reduction in IOP at post-bsl visits;

data iop;
    set site_subj2;
    do i = 1 to 4; ** 4 visits;
	   do j = 1 to 3; ** 3 timepoints/visit;
	     visit = i;
	     hour = j;
	     if i = 1  then do;  ** Baseline;
		     if j =1 then iop = round((RANNOR(&seed. + i + j)* &SD. + 25), .1); ** Hour 0;
		     if j =2 then iop = round((RANNOR(&seed. + i + j)* &SD. + 23), .1); ** Hour 2;
		     if j =3 then iop = round((RANNOR(&seed. + i + j)* &SD. + 22), .1); ** Hour 8;
	     end;
	     else if i > 1 then do; ** Post-baseline: with about 6 to 8 mmHg reduction in IOP;
		     if j =1 then iop = round((RANNOR(&seed. + i + j)* &SD. + 17.5), .1); ** Hour 0;
		     if j =2 then iop = round((RANNOR(&seed. + i + j)* &SD. + 16.5), .1); ** Hour 2;
		     if j =3 then iop = round((RANNOR(&seed. + i + j)* &SD. + 16.2), .1); ** Hour 8;
       end;
       output;
	  end; 
	end;
	
  drop i j shuffle;
  format visit visdf. hour hrdf. ;
run;


proc sort data=iop;
  by siteid subjid visit hour;
run;


** Mean diurnal IOP at each visit: mean of hours 0, 2 and 8 average eye IOP at baseline;
proc transpose data=iop out=iop_t ;
  by siteid subjid visit;
  var iop;
  id hour;
run;

data iop_MnDiur;
  set iop_t;
  iop_MnDiur = round (mean (Hour_0, Hour_2, Hour_8), .01);
  drop _NAME_ ;
run;


** Set the subject index number to be used in X-axis;
data subjid;
  set site_subj2;
  subj_id = _N_;
  if siteid = 1 then subj_index = _N_;  
  if siteid = 2 then subj_index = _N_ + 1; ** leave space for reference line;
  if siteid = 3 then subj_index = _N_ + 2;
  if siteid = 4 then subj_index = _N_ + 3;
  if siteid = 5 then subj_index = _N_ + 4;
  if siteid = 6 then subj_index = _N_ + 5;
  if siteid = 7 then subj_index = _N_ + 6;
  if siteid = 8 then subj_index = _N_ + 7;
  if siteid = 9 then subj_index = _N_ + 8;
  if siteid = 10 then subj_index = _N_ + 9;
  keep siteid subjid subj_index;
run;


** Set macro variables to hold the reference line locations in x-axis;
data _NULL_;
  refn1 = &n_site1. + 1; 
  refn2 = refn1 + &n_site2. + 1; 
  refn3 = refn2 + &n_site3. + 1;
  refn4 = refn3 + &n_site4. + 1; 
  refn5 = refn4 + &n_site5. + 1;
  refn6 = refn5 + &n_site6. + 1;
  refn7 = refn6 + &n_site7. + 1;
  refn8 = refn7 + &n_site8. + 1;
  refn9 = refn8 + &n_site9. + 1;
  refn10 = refn9 + &n_site10. + 1;

  call symput ("Ref1", refn1);
  call symput ("Ref2", refn2);
  call symput ("Ref3", refn3);
  call symput ("Ref4", refn4);
  call symput ("Ref5", refn5);
  call symput ("Ref6", refn6);
  call symput ("Ref7", refn7);
  call symput ("Ref8", refn8);
  call symput ("Ref9", refn9);
  call symput ("Ref10", refn10);
run;

data mndiur_all;
  merge iop_MnDiur subjid;
  by siteid subjid;
run;

proc transpose data=MnDiur_all out=MnDiur_T prefix=vst_;
  by siteid subjid subj_index;
  var iop_MnDiur;
run;

data  mndiur_anno;
  set MnDiur_T;  
  drop _NAME_ ;
  rename vst_1 = BSL vst_2 = WK4 vst_3 = WK8 vst_4 = WK12;
run;

proc format;
  value siteid
    1 = '1'
    &ref1. = '2'
    &ref2. = '3'
    &ref3. = '4'
    &ref4. = '5'
    &ref5. = '6'
    &ref6. = '7'
    &ref7. = '8'
    &ref8. = '9'
    &ref9. = '10'
    OTHER = ' ';
run;

** Annotate dataset to connect the 2 IOPs of the same subject by a straight line;
data anno1;   
  set mndiur_anno;
  by subj_index;
   function='move'; xsys='2'; ysys='2'; x=subj_index; y=bsl; output;
   function='draw'; xsys='2'; ysys='2'; color='VILG'; x=subj_index; y=wk4; size=2; line=1;
   output;
   drop wk8 wk12;
run;
data anno2;   
  set mndiur_anno;
  by subj_index;
   function='move'; xsys='2'; ysys='2'; x=subj_index; y=bsl; output;
   function='draw'; xsys='2'; ysys='2'; color='VILG'; x=subj_index; y=wk8; size=2; line=1;
   output;
   drop wk4 wk12;
run;
data anno3;   
  set mndiur_anno;
  by subj_index;
   function='move'; xsys='2'; ysys='2'; x=subj_index; y=bsl; output;
   function='draw'; xsys='2'; ysys='2'; color='VILG'; x=subj_index; y=wk12; size=2; line=1;
   output;
   drop wk4 wk8;
run;


%macro Thunderstorm_GPLOT (driver=, ext=, fontname=);

** Set the fonts and formats for titles/footnotes; 
%LET FMTTITLE  =H=2.5 JUSTIFY=CENTER FONT="&FONTNAME";
%LET FMTFOOT =H=2   JUSTIFY=CENTER FONT="&FONTNAME";

goptions
  reset  = all           
  GUNIT=PCT
  rotate  = landscape        
  gsfmode = replace          
  gsfname = GSASFILE         /* OUTPUT FILE REFERENCE */
  device  = &DRIVER          
  hsize   = 8 in             
  horigin = 0 in           
  vsize   = 6.5 in           
  vorigin = 0 in          
  ftext   = "&FONTNAME"    
  htext   = 10pt           
  ftitle  = "&FONTNAME"    
  htitle  = 10pt     
 ;

ods proclabel="Thunderstorm Scatter Plot for Mean Diurnal IOP"; ** upper-level bookmark;

SYMBOL1 H=2.5 C=RED CO=RED INTERPOL=none W=2 L=1 VALUE=circle;
SYMBOL2 H=2.5 C=BLACK CO=BLACK INTERPOL=none W=2 L=2 VALUE=dot;

AXIS1  MINOR=NONE OFFSET=(1,1) order=(10 to 30 by 5) LABEL=(FONT="&FONTNAME" ANGLE=90 HEIGHT=2.5 font="&FONTNAME" "Mean Diurnal IOP at Baseline and Post-baseline (mm Hg)" ) 
	   VALUE=(H=1.5) ; 
AXIS2  MAJOR=NONE MINOR=NONE OFFSET=(1,1) order=(0 to 110 by 1) LABEL=(FONT="&FONTNAME"  HEIGHT=2.5 font="&FONTNAME" "Subject Grouped by Investigator Sites") 
       VALUE=(H=2) reflabel=(position=bottom c= blue font="&FONTNAME" h=2 j=r	
		"(n=&n_site1.))" "(n=&n_site2.)" "(n=&n_site3.)" "(n=&n_site4.)" "(n=&n_site5.)"
		"(n=&n_site6.)" "(n=&n_site7.)" "(n=&n_site8.)" "(n=&n_site9.)" "(n=&n_site10.)");

proc gplot data= mndiur_all;  
  plot 	iop_MnDiur * subj_index = visit / HAXIS=AXIS2 VAXIS=AXIS1 noframe anno=anno1 des= "Baseline and Week 4"
			href=0 &ref1. &ref2. &ref3. &ref4. &ref5. &ref6. &ref7. &ref8. &ref9. &ref10.;   

  FILENAME GSASFILE "&OUTLOC./Thunderstorm_Fig1.&EXT.";
  title1 &FMTTITLE "Figure 1. Thunderstorm Scatter Plot for Mean Diurnal IOP";
  title2 &FMTTITLE "Baseline and Week 4";
  footnote1 &FMTFOOT "&pgmpth.";
  where visit in (1, 2);
  label visit = 'Legend:'; format subj_index siteid.;
  run;	

  plot 	iop_MnDiur * subj_index = visit / HAXIS=AXIS2 VAXIS=AXIS1 noframe anno=anno2 des= "Baseline and Week 8"
			href=0 &ref1. &ref2. &ref3. &ref4. &ref5. &ref6. &ref7. &ref8. &ref9. &ref10.;  

  FILENAME GSASFILE "&OUTLOC./Thunderstorm_Fig2.&EXT.";
  title1 &FMTTITLE "Figure 2. Thunderstorm Scatter Plot for Mean Diurnal IOP";
  title2 &FMTTITLE "Baseline and Week 8";	
  footnote1 &FMTFOOT "&pgmpth.";
  where visit in (1, 3);	
  label visit = 'Legend:'; format subj_index siteid.;
  run;

  plot 	iop_MnDiur * subj_index = visit / HAXIS=AXIS2 VAXIS=AXIS1 noframe anno=anno3 des= "Baseline and Week 12"
			href=0 &ref1. &ref2. &ref3. &ref4. &ref5. &ref6. &ref7. &ref8. &ref9. &ref10.;  
  FILENAME GSASFILE "&OUTLOC./Thunderstorm_Fig3.&EXT.";

  title1 &FMTTITLE "Figure 3. Thunderstorm Scatter Plot for Mean Diurnal IOP";
  title2 &FMTTITLE "Baseline and Week 12";
  footnote1 &FMTFOOT "&pgmpth.";
  where visit in (1, 4);	
  label visit = 'Legend:'; format subj_index siteid.;
  run;
quit;

%mend Thunderstorm_GPLOT;


options orientation=landscape;
ods listing close;
** Produce figures in EPS format and save into an PDF file;
ods pdf file= "&outloc./Thunderstorm_GPLOT.pdf" ;
%Thunderstorm_GPLOT (driver=pscolor, ext=eps, fontname=Times-Roman);
ods pdf close;


** Produce figures in EMF format and save into an RTF file;
ods rtf file= "&outloc./Thunderstorm_GPLOT.rtf" style=minimal nogtitle nogfootnote;
%Thunderstorm_GPLOT (driver=sasemf, ext=emf, fontname=Courier New);
ods rtf close;
ods listing;


** Re-produce the figures in ODS Graphics - using SGPLOT;

proc transpose data=mndiur_all out=mndiur_all2 ;
  by subjid subj_index;
  var iop_MnDiur;
  id visit;
run;


%macro Thunderstorm_SGPLOT (OUTPUTFMT=, FONTNAME=);

  %LET FMTTITLE  =H=1.25 JUSTIFY=CENTER FONT="&FONTNAME";
  %LET FMTFOOT =H=1   JUSTIFY=CENTER FONT="&FONTNAME";

  ods listing gpath="&outloc.";
  ods graphics / reset=all width=8in height=6in noborder OUTPUTFMT=&OUTPUTFMT. imagename="Thunderstorm_SG";

  title1 &FMTTITLE "Figure 1. Thunderstorm Scatter Plot for Mean Diurnal IOP";
  title2 &FMTTITLE "Baseline and Week 4";
  footnote1 &FMTFOOT "&pgmpth.";
  ods proclabel="Thunderstorm Scatter Plot for Mean Diurnal IOP at Baseline and Week 4";  
  proc sgplot data=mndiur_all2  noautolegend;
    scatter x=subj_index y=baseline  /  markerattrs= (symbol=circle color=red size=10) name="BSL" legendlabel="Baseline";
    scatter x=subj_index y=week_4  /  markerattrs= (symbol=circlefilled color=black size=10) name="PBSL" legendlabel="Week 4";	
    highlow x=subj_index high=week_4 low=baseline /	lineattrs = (thickness = 2 PATTERN=solid COLOR=green) ;					

    xaxis VALUES= (0 to 110 by 1) label = "Subject Grouped by Investigator Sites" display = (noticks);
    yaxis VALUES= (10 to 30 by 5) label = "Mean Diurnal IOP at Baseline and Week 4 (mm Hg)";
	REFLINE 0 &ref1. &ref2. &ref3. &ref4. &ref5. &ref6. &ref7. &ref8. &ref9. &ref10. /AXIS=x 
      LABEL = ("(n=&n_site1.)" "(n=&n_site2.)" "(n=&n_site3.)" "(n=&n_site4.)" "(n=&n_site5.)"
			  "(n=&n_site6.)" "(n=&n_site7.)" "(n=&n_site8.)" "(n=&n_site9.)" "(n=&n_site10.)");

	format subj_index siteid.;
    keylegend "BSL" "PBSL" / noborder title = 'Legend:';
  run;
  quit; 
		

  title1 &FMTTITLE "Figure 2. Thunderstorm Scatter Plot for Mean Diurnal IOP";
  title2 &FMTTITLE "Baseline and Week 8";
  footnote1 &FMTFOOT "&pgmpth.";
  ods proclabel="Thunderstorm Scatter Plot for Mean Diurnal IOP at Baseline and Week 8";  
  proc sgplot data=mndiur_all2  noautolegend;
    scatter x=subj_index y=baseline  /  markerattrs= (symbol=circle color=red size=10) name="BSL" legendlabel="Baseline";
    scatter x=subj_index y=week_8  /  markerattrs= (symbol=circlefilled color=black size=10) name="PBSL" legendlabel="Week 8";	
    highlow x=subj_index high=week_8 low=baseline /	lineattrs = (thickness = 2 PATTERN=solid COLOR=green) ;					

    xaxis VALUES= (0 to 110 by 1) label = "Subject Grouped by Investigator Sites" display = (noticks);
    yaxis VALUES= (10 to 30 by 5) label = "Mean Diurnal IOP at Baseline and Week 8 (mm Hg)";
	REFLINE 0 &ref1. &ref2. &ref3. &ref4. &ref5. &ref6. &ref7. &ref8. &ref9. &ref10. /AXIS=x 
      LABEL = ("(n=&n_site1.)" "(n=&n_site2.)" "(n=&n_site3.)" "(n=&n_site4.)" "(n=&n_site5.)"
			  "(n=&n_site6.)" "(n=&n_site7.)" "(n=&n_site8.)" "(n=&n_site9.)" "(n=&n_site10.)");
    format subj_index siteid.;
    keylegend "BSL" "PBSL" / noborder title = 'Legend:';
  run;
  quit;

  title1 &FMTTITLE "Figure 3. Thunderstorm Scatter Plot for Mean Diurnal IOP";
  title2 &FMTTITLE "Baseline and Week 12";
  footnote1 &FMTFOOT "&pgmpth.";
  ods proclabel="Thunderstorm Scatter Plot for Mean Diurnal IOP at Baseline and Week 12";  
  proc sgplot data=mndiur_all2  noautolegend;
    scatter x=subj_index y=baseline  /  markerattrs= (symbol=circle color=red size=10) name="BSL" legendlabel="Baseline";
    scatter x=subj_index y=week_8  /  markerattrs= (symbol=circlefilled color=black size=10) name="PBSL" legendlabel="Week 12";	
    highlow x=subj_index high=week_8 low=baseline /	lineattrs = (thickness = 2 PATTERN=solid COLOR=green) ;					

    xaxis VALUES= (0 to 110 by 1) label = "Subject Grouped by Investigator Sites" display = (noticks);
    yaxis VALUES= (10 to 30 by 5) label = "Mean Diurnal IOP at Baseline and Week 12 (mm Hg)";
	REFLINE 0 &ref1. &ref2. &ref3. &ref4. &ref5. &ref6. &ref7. &ref8. &ref9. &ref10. /AXIS=x 
      LABEL = ("(n=&n_site1.)" "(n=&n_site2.)" "(n=&n_site3.)" "(n=&n_site4.)" "(n=&n_site5.)"
			  "(n=&n_site6.)" "(n=&n_site7.)" "(n=&n_site8.)" "(n=&n_site9.)" "(n=&n_site10.)");
	format subj_index siteid.;
    keylegend "BSL" "PBSL" / noborder title = 'Legend:';
  run;
  quit;
%mend Thunderstorm_SGPLOT;


** Produce figures in PS format and save into an PDF file;
ods listing close;
ods pdf file= "&outloc./Thunderstorm_SGPLOT.pdf" ;
  %Thunderstorm_SGPLOT (OUTPUTFMT=ps, FONTNAME=Times-Roman);
ods pdf close;

** Produce figures in EMF format and save into an RTF file;
ods rtf file= "&outloc./Thunderstorm_SGPLOT.rtf" nogtitle nogfootnote;
  %Thunderstorm_SGPLOT (OUTPUTFMT=emf, FONTNAME=Courier New);
ods rtf close;
ods listing;


